/*
** Module   : primes.p
** Abstract : calculates all prime numbers between (and including two integer
**            values)
**
** Copyright (c) 2005-2019, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 GES 20050609          First version using the fermat test, which 
**                           artificially limits the maximum integral value
**                           tested as 31.
** 002 GES 20070307          Reworked to have a more modular structure and to
**                           use a wider range of user-interface and database
**                           database language features.
** 003 GES 20100726          Added an unnecessary temp-table index to allow the
**                           resulting output to appear as a dmo_index example.
** 004 ECF 20160124          Refactored to use primes.i include file.
** 005 ECF 20160126          Reworked to run external procedure and use a shared
**                           temp-table and shared variable.
** 006 RFB 20190829          Created a pre-processor directive to set the max
**                           value at 31, and check for that before proceeding.
**                           This prevents an error popup that occurs until the 
**                           max value is looped to.
*/

{primes.i &share_opt="new"}

def var start-num as int init 0.
def var end-num   as int init {&MAX-VAL}.
def var continue  as logical init true.

repeat while continue on error undo, leave:
   run edit-range.
   run calc-primes.
   run display-primes.p (buffer list).
   message "Try Again?" update continue.
end.

/*
 * Prompt the user for the range of integers to test.  A start and end
 * value will be edited with the previous value set as a default. The start
 * value must be non-negative and the end value must be larger than the
 * start value.
 */
procedure edit-range:

   form start-num label "Range Start"
        end-num   label "Range End"
        with frame range-parms side-labels title "Edit Test Range".
     
   update start-num validate(start-num >= 0, "Start of range must be non-negative.")
          end-num   validate((end-num > start-num) and (end-num <= {&MAX-VAL}), 
                             "End of range must be larger than start of range and less than or equal to {&MAX-VAL}.")
          with frame range-parms.
end.

/*
 * Implements the fermat test as a mechanism for detecting if an integer is
 * a prime number. This test is flawed in the following ways:
 *
 * 1. It only tests the probability that a number is prime. With the current 
 *    implementation, it is correct on the initial range of integers that
 *    can be tested, so this limitation is theoretical only.
 * 2. This implementation raises 2 to the power of the given integer minus 1.
 *    The result of this exponentiation rapidly exceeds the capacity of a
 *    32-bit integer to contain it. Thus this implementation of the algorithm
 *    is only useful for calculating primes between 1 and 31.  Quite limited
 *    but OK for now.
 */
function fermatTest returns logical (input x as int).
   return (exp(2, (x - 1)) modulo x) eq 1.
end.

/*
 * Create an entry in the temp table for every prime number found between
 * the start and end of the range (inclusive).
 */
procedure calc-primes:

   def var possible as int.
   def var actual   as logical.
   
   empty temp-table list.
   
   prime-count = 0.

   repeat possible = start-num to end-num
          on error undo, leave:

      actual = false.

      if possible < 3 then
         actual = true.
      else
         /* even numbers greater than 2 can't be prime! */
         actual = (possible modulo 2) <> 0 and fermatTest(possible).

      if actual then
      do:
         create list.
         list.nu = possible.
         prime-count = prime-count + 1.
      end.
   end.
   
end.
